@@ -1,4 +1,4 @@
-/* $Id: bogomain.c,v 1.13 2005/04/04 11:16:19 relson Exp $ */
+/* $Id: bogomain.c,v 1.17 2005/09/05 19:06:37 relson Exp $ */
 
 /*****************************************************************************
 
@@ -40,7 +40,8 @@ ex_t bogomain(int argc, char **argv) /*@globals errno,stderr,stdout@*/
 {
     rc_t status;
     ex_t exitcode = EX_OK;
-    bfpath_mode mode;
+
+    fBogofilter = true;
 
     dbgout = stderr;
 
@@ -55,17 +56,12 @@ ex_t bogomain(int argc, char **argv) /*@globals errno,stderr,stdout@*/
 	openlog("bogofilter", LOG_PID, LOG_MAIL);
 #endif
 
-    if (((run_type & UNREG_SPAM) != 0) ||
-	((run_type & UNREG_GOOD) != 0))
-	mode = BFP_MUST_EXIST;
-    else
-	mode = BFP_MAY_CREATE;
-
-    bfpath_set_mode(mode);
-
     /* open all wordlists */
     open_wordlists((run_type == RUN_NORMAL) ? DS_READ : DS_WRITE);
 
+    if (encoding == E_UNKNOWN)
+	encoding = E_DEFAULT;
+
     status = bogofilter(argc - optind, argv + optind);
 
     switch (status) {
@@ -86,13 +82,11 @@ ex_t bogomain(int argc, char **argv) /*@globals errno,stderr,stdout@*/
     if (DEBUG_MEMORY(1))
 	MEMDISPLAY;
 
-    free_wordlists();
-
-    output_cleanup();
-
     /* cleanup storage */
-    mime_cleanup();
     token_cleanup();
+    mime_cleanup();
+    output_cleanup();
+    free_wordlists();
 
 #ifdef	HAVE_SYSLOG_H
     if (logflag)
