@@ -1,4 +1,4 @@
-/* $Id: bogoreader.c,v 1.62 2004/10/02 12:55:17 relson Exp $ */
+/* $Id: bogoreader.c,v 1.66 2005/09/04 14:49:43 relson Exp $ */
 
 /*****************************************************************************
 
@@ -53,7 +53,6 @@ static ms_t mailstore_type;
 static bool mail_first = true;		/* for the _next_mail functions */
 static bool mailstore_first = true;	/* for the _next_mailstore functions */
 static bool firstline = true;		/* for mailbox /^From / match */
-static bool emptyline = false;		/* for mailbox /^From / match */
 
 static bool    have_message = false;
 
@@ -360,13 +359,13 @@ static bool dir_next_mail(void)
     struct dirent *dirent;
     struct stat st;
 
-    for(;;) {
+    for (;;) {
 	if (reader_dir == NULL) {
 	    char *x = dir_name;
 	    /* open next directory */
 	    if (mailstore_type == MS_MAILDIR) {
 		size_t siz;
-		
+
 		if (*maildir_sub == NULL)
 		    return false; /* IMPORTANT for termination */
 		siz = strlen(dir_name) + 4 + 1;
@@ -383,13 +382,19 @@ static bool dir_next_mail(void)
 		xfree(x);
 	}
 
-	while ((dirent = readdir(reader_dir)) != NULL) {
+	while ((errno = 0, dirent = readdir(reader_dir)) != NULL) {
 	    /* skip private files */
 	    if ((mailstore_type == MS_MAILDIR && dirent->d_name[0] != '.') ||
 		(mailstore_type == MS_MH && isdigit((unsigned char)dirent->d_name[0])))
 		break;
 	}
 
+	if (errno) {
+	    fprintf(stderr, "Cannot read directory %s: %s",
+		    dir_name, strerror(errno));
+	    exit(EX_ERROR);
+	}
+
 	if (dirent == NULL) {
 	    if (reader_dir)
 		closedir(reader_dir);
@@ -436,6 +441,7 @@ static int mailbox_getline(buff_t *buff)
     byte *buf = buff->t.text + used;
     int count;
     static word_t *saved = NULL;
+    static bool emptyline = false;		/* for mailbox /^From / match */
 
     if (saved != NULL) {
 	count = saved->leng;
@@ -523,8 +529,6 @@ static int rmail_getline(buff_t *buff)
 	    Z(buff->t.text[buff->t.leng]);	/* for easier debugging - removable */
     }
 
-    emptyline = is_eol((char *)buf, count);
-
     return count;
 }
 
@@ -563,13 +567,11 @@ static int ant_getline(buff_t *buff)
         if ((count == 2 || count == 3) && 
 	    (buf[0] == '.') && 
 	    (buf[1] == '\r' || buf[1] == '\n'))
-            dot_found = true;		/* dot found.  look for separator */
+            dot_found = true;			/* dot found.  look for separator */
 	if (buff->t.leng < buff->size)		/* for easier debugging - removable */
 	    Z(buff->t.text[buff->t.leng]);	/* for easier debugging - removable */
     }
 
-    emptyline = is_eol((char *)buf, count);
-
     return count;
 }
 
