@@ -1,4 +1,4 @@
-/* $Id: bogotune.c,v 1.223 2005/06/30 00:07:59 relson Exp $ */
+/* $Id: bogotune.c,v 1.226 2005/09/04 14:49:43 relson Exp $ */
 
 /*****************************************************************************
 
@@ -46,6 +46,7 @@ AUTHORS:
 #include <ctype.h>
 #include <errno.h>
 #include <stdlib.h>
+#include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 
@@ -137,7 +138,6 @@ typedef enum e_ds_loc {
 
 const char *progname = "bogotune";
 static char *ds_path;			/* directory */
-static char *ds_file;			/* directory/file */
 static ds_loc ds_flag = DS_NONE;
 static void *env = NULL;
 
@@ -629,6 +629,18 @@ static int update_count(void)
     return message_count;
 }
 
+static unsigned int calc_db_cachesize(void)
+{
+    struct stat fst;
+    if (!stat(ds_path, &fst)) {
+	int dbc = ceil((double)fst.st_size / (3 * 1024 * 1024));
+        return ((unsigned int)dbc);
+    } else {
+        fprintf(stderr, "Unable to stat %s\n", ds_path);
+        exit (EX_ERROR);
+    }
+}
+
 static void load_wordlist(ds_foreach_t *hook, void *userdata)
 {
     bfpath *bfp = bfpath_create(ds_path);
@@ -1066,7 +1078,7 @@ static void top_ten(result_t *sorted, uint n)
     printf("Top ten parameter sets from this scan:\n");
 
     printf("        rs     md    rx    spesf    nsesf    co     fp  fn   fppc   fnpc\n");
-    for(f = false; !f; f = true) {
+    for (f = false; !f; f = true) {
       for (i = j = 0; i < 10 && j < n;) {
  	result_t *r = &sorted[j++];
  	if (!f && r->fp != target) continue;
@@ -1216,7 +1228,7 @@ static void final_recommendations(bool skip)
     printf("Non-Spam...\n");
     score_ns(ns_scores);	/* get scores (in descending order) */
 
-    for(m=0; m<10; ++m) printf("%8.6f %8.6f\n", sp_scores[m], ns_scores[m]);
+    for (m=0; m<10; ++m) printf("%8.6f %8.6f\n", sp_scores[m], ns_scores[m]);
 
     if (verbose >= PARMS)
 	printf("# ns_cnt %u, sp_cnt %u\n", ns_cnt, sp_cnt);
@@ -1331,7 +1343,6 @@ static void bogotune_free(void)
     mime_cleanup();
 
     xfree(ds_path);
-    xfree(ds_file);
 
     return;
 }
@@ -1477,13 +1488,15 @@ static rc_t bogotune(void)
 	scoring_error();
 
     /*
-    ** 5.  Calculate x
+    ** 5.  Calculate x and cache size
     ** Calculate x with bogoutil's -r option (a new addition).
     ** Bound the calculated value within [0.4, 0.6] and set the range to be
     ** investigated to [x-0.1, x+0.1].
     */
 
     robx = get_robx();
+    db_cachesize = calc_db_cachesize();
+    printf("Recommended db cache size is %u MB\n", db_cachesize);
 
     /*
     ** 6.  Calculate fp target
