@@ -1,4 +1,4 @@
-/* $Id: common.h,v 1.57 2005/03/17 12:35:14 m-a Exp $ */
+/* $Id: common.h,v 1.63 2005/09/11 12:51:52 relson Exp $ */
 
 /*****************************************************************************
 
@@ -131,6 +131,31 @@ typedef enum bulk_e {
     B_STDIN
 } bulk_t;
 
+/* for transaction flag */
+
+typedef	enum {
+    T_ERROR    = -1,	/* -1 for error */
+    T_DISABLED =  0,	/*  0 for no transactions - 0 must mean T_DISABLE
+			    for compatibility with dummy functions */
+    T_ENABLED  =  1,	/*  1 for transactions */
+    T_DEFAULT_OFF =  2, /*  2 for off, unless explicity specified */
+    T_DEFAULT_ON  =  3, /*  3 for on, unless explicity specified */
+    T_DONT_KNOW		/*  4 for don't know */
+} e_txn;
+
+/* for encoding (unicode) flag */
+
+typedef	enum {
+    E_UNKNOWN = 0,	/* 0 for not set  */
+    E_RAW     = 1,	/* 1 for raw text */
+    E_UNICODE,		/* 2 for unicode  */
+#ifndef	ENABLE_UNICODE
+    E_DEFAULT = E_RAW
+#else
+    E_DEFAULT = E_UNICODE
+#endif
+} e_enc;
+
 #include "globals.h"
 
 /* Represents the secondary data for a word key */
@@ -147,11 +172,13 @@ typedef struct {
     int freq;
 } wordprop_t;
 
+extern void bf_exit(void);
+
 #define internal_error do { fprintf(stderr, "Internal error in %s:%lu\n", __FILE__, (unsigned long)__LINE__); abort(); } while(0)
 
 typedef enum e_wordlist_version {
     ORIGINAL_VERSION = 0,
-    IP_PREFIX = 20040500
+    IP_PREFIX = 20040500	/* when IP prefixes were added */
 } t_wordlist_version;
 
 #define	CURRENT_VERSION	IP_PREFIX
@@ -159,6 +186,7 @@ typedef enum e_wordlist_version {
 /* for bogoutil.c and datastore_db_trans.c */
 
 typedef enum { M_NONE, M_DUMP, M_LOAD, M_WORD, M_MAINTAIN, M_ROBX, M_HIST,
+    M_LIST_LOGFILES, M_LEAFPAGES,
     M_RECOVER, M_CRECOVER, M_PURGELOGS, M_VERIFY, M_REMOVEENV, M_CHECKPOINT,
     M_PAGESIZE }
     cmd_t;
