@@ -1,4 +1,4 @@
-/* $Id: datastore.h,v 1.51 2005/04/04 11:48:49 relson Exp $ */
+/* $Id: datastore.h,v 1.58 2005/06/17 03:33:22 relson Exp $ */
 
 /*****************************************************************************
 
@@ -24,7 +24,7 @@ Matthias Andree <matthias.andree@gmx.de> 2003
 
 #include <stdlib.h>
 
-#ifdef	ENABLE_DB_DATASTORE	/* if not Berkeley DB */
+#ifdef	ENABLE_DB_DATASTORE	/* if Berkeley DB */
 #include <db.h>
 #endif
 
@@ -113,6 +113,7 @@ typedef ex_t	dsm_x_pp	(bfpath *bfp);
 typedef u_int32_t dsm_u_pp	(bfpath *bfp);
 typedef DB_ENV *dsm_pnv_pp	(bfpath *bfp);
 typedef DB_ENV *dsm_pnv_pbe	(dbe_t *env);
+typedef ex_t	dsm_x_ppsi	(bfpath *bfp, int argc, char **argv);
 
 /** Datastore methods type, used by datastore/database layers to switch
  * implementations after detection of database type. */
@@ -140,6 +141,8 @@ typedef struct {
     dsm_x_ppbb	 *dsm_recover;
     dsm_x_pp	 *dsm_remove;
     dsm_x_pp	 *dsm_verify;
+    dsm_x_ppsi	 *dsm_list_logfiles;
+    dsm_u_pp	 *dsm_leafpages;
 } dsm_t;
 
 extern dsm_t *dsm;
@@ -290,6 +293,12 @@ extern int ds_get_wordlist_version(void *vhandle, dsv_t *val);
 /** set the database version */
 extern int ds_set_wordlist_version(void *vhandle, dsv_t *val);
 
+/** Get the database encoding */
+extern int ds_get_wordlist_encoding(void *vhandle, dsv_t *val);
+
+/** set the database encoding */
+extern int ds_set_wordlist_encoding(void *vhandle, int enc);
+
 /** Get the current process ID. */
 extern unsigned long ds_handle_pid(void *vhandle);
 
@@ -311,6 +320,10 @@ extern ex_t ds_remove(bfpath *bfp);
 /** Verify given database */
 extern ex_t ds_verify(bfpath *bfp);
 
+/** Return leaf page count of given database, \return 0xffffffff for error,
+ * 0 if unknown. */
+extern u_int32_t ds_leafpages(bfpath *bfp);
+
 /** Return page size of given database, \return 0xffffffff for error, 0 for
  * variable sized pages or unpaged datastores. */
 extern u_int32_t ds_pagesize(bfpath *bfp);
@@ -324,4 +337,15 @@ extern ex_t ds_checkpoint(bfpath *bfp);
 /** datastore backends must provide this initializing function */
 extern void dsm_init(bfpath *bfp);
 
+/** list log files, remaining arguments are backend specific */
+extern ex_t ds_list_logfiles(bfpath *bfp, int argc, char **argv);
+
+/** parse bogofilter option
+ * \return true if an option was recognized, false otherwise */
+extern bool dsm_options_bogofilter(int option, const char *name, const char *val);
+
+/** parse bogoutil option
+ * \return true if an option was recognized, false otherwise */
+extern bool dsm_options_bogoutil(int option, cmd_t *flag, int *count, const char **ds_file, const char *name, const char *val);
+
 #endif
