@@ -1,4 +1,4 @@
-/* $Id: datastore_db.c,v 1.200 2005/04/07 08:19:25 m-a Exp $ */
+/* $Id: datastore_db.c,v 1.221 2005/11/23 22:56:23 m-a Exp $ */
 
 /*****************************************************************************
 
@@ -7,7 +7,7 @@ datastore_db.c -- implements the datastore, using Berkeley DB
 
 AUTHORS:
 Gyepi Sam <gyepi@praxis-sw.com>   2002 - 2003
-Matthias Andree <matthias.andree@gmx.de> 2003 - 2004
+Matthias Andree <matthias.andree@gmx.de> 2003 - 2005
 
 ******************************************************************************/
 
@@ -22,9 +22,6 @@ Matthias Andree <matthias.andree@gmx.de> 2003 - 2004
  *   deadlock
  * - document code changes
  * - conduct massive tests
- * - check if we really need the log files for "catastrophic recovery"
- *   or if we can remove them (see db_archive documentation)
- *   as the log files are *HUGE* even compared with the data base
  */
 
 /*
@@ -32,7 +29,6 @@ Matthias Andree <matthias.andree@gmx.de> 2003 - 2004
  * in the many different BerkeleyDB versions.
  */
 
-#define DONT_TYPEDEF_SSIZE_T 1
 #include "common.h"
 
 #include <errno.h>
@@ -67,9 +63,6 @@ dsm_t *dsm = &dsm_transactional;
 
 /** Default flags for DB_ENV->open() */
 
-u_int32_t db_max_locks = 16384;		/* set_lk_max_locks    32768 */
-u_int32_t db_max_objects = 16384;	/* set_lk_max_objects  32768 */
-
 bool	  db_log_autoremove = true;	/* DB_LOG_AUTOREMOVE */
 
 #ifdef	FUTURE_DB_OPTIONS
@@ -95,6 +88,62 @@ static const char *resolveopenflags(u_int32_t flags) {
     return buf;
 }
 
+static void set_txn_mode(void)
+{
+    if (eTransaction == T_DEFAULT_OFF)
+	eTransaction =  T_DISABLED;
+    if (eTransaction == T_DEFAULT_ON)
+	eTransaction =  T_ENABLED;
+}
+
+#if !defined(ENABLE_TRANSACTIONS) && !defined(DISABLE_TRANSACTIONS)
+static const char *txn2str(e_txn txn)
+{
+    const char *v = "unknown";
+    switch (txn) {
+    case T_ERROR: 	v = "T_ERROR";		break;
+    case T_DISABLED: 	v = "T_DISABLED";	break;
+    case T_ENABLED: 	v = "T_ENABLED";	break;
+    case T_DEFAULT_OFF:	v = "T_DEFAULT_OFF";	break;
+    case T_DEFAULT_ON: 	v = "T_DEFAULT_ON";	break;
+    case T_DONT_KNOW:	v = "T_DONT_KNOW";	break;
+    }
+    return v;
+}
+#endif
+
+#if !defined(ENABLE_TRANSACTIONS) && !defined(DISABLE_TRANSACTIONS)
+static e_txn get_txn_state(bfpath *bfp)
+{
+    e_txn txn = eTransaction;
+
+    /* check for overrides (for test suite, etc.) */
+    if (getenv("BF_FORCE_NON_TRANSACTIONAL"))
+	txn = T_DISABLED;
+    if (getenv("BF_FORCE_TRANSACTIONAL"))
+	txn = T_ENABLED;
+
+    if (txn != T_DISABLED && txn != T_ENABLED) {
+	/* if not set, probe for database environment */
+	e_txn probe;
+
+	if (DEBUG_DATABASE(1))
+	    fprintf(dbgout, "probing \"%s\" and \"%s\" for environment...",
+		    bfp->dirname, bfp->filename);
+
+	probe = probe_txn(bfp);
+
+	if (probe == T_DISABLED || probe == T_ENABLED)
+	    txn = probe;
+
+	if (DEBUG_DATABASE(1))
+	    fprintf(dbgout, "%s\n", txn2str(probe));
+    } /* else just use the default */
+
+    return txn;
+}
+#endif
+
 static void check_env_pagesize(DB *db)
 {
     const char *ps = getenv("BF_PAGESIZE");
@@ -128,7 +177,7 @@ static int DB_OPEN(DB *db, bfpath *bfp, const char *database, DBTYPE type, u_int
 	flags |= dsm->dsm_auto_commit_flags();
 #endif
 
-    if (!fTransaction)
+    if (eTransaction == T_DISABLED)
 	file = bfp->filepath;
     else
 	file = bfp->filename;
@@ -192,7 +241,7 @@ void *db_get_env(void *vhandle)
 }
 
 /* implements locking. */
-int db_lock(int fd, int cmd, short int type)
+int subr_db_lock(int fd, int cmd, short int type)
 {
     struct flock lock;
 
@@ -203,42 +252,48 @@ int db_lock(int fd, int cmd, short int type)
     return (fcntl(fd, cmd, &lock));
 }
 
+int db_lock(void *vhandle, int open_mode)
+{
+    int e = 0;
+    dbh_t *handle = vhandle;
+
+    /* try fcntl lock */
+    handle->locked = false;
+    if (subr_db_lock(handle->fd, F_SETLK,
+		(short int)(open_mode == DS_READ ? F_RDLCK : F_WRLCK)))
+    {
+	e = errno;
+	db_close(handle);
+	errno = e;
+	if (errno == EACCES)
+	    e = errno = EAGAIN;
+    } else {
+	/* have lock */
+	if (handle->fd >= 0)
+	    handle->locked = true;
+    }
+    return e;
+}
+
 void dsm_init(bfpath *bfp)
 {
     /* Note: we assume that the default dsm initializers above handle
      * this properly, dsm_init is empty if transactions are disabled or
      * forced. */
 #if !defined(ENABLE_TRANSACTIONS) && !defined(DISABLE_TRANSACTIONS)
-    /* we neither force nor forbid transactions, so add probe code */
-    probe_txn_t txn;
 
-    if (DEBUG_DATABASE(2))
-	fprintf(dbgout, "probing \"%s\" and \"%s\" for environment...\n",
-		bfp->dirname, bfp->filename);
+    eTransaction = get_txn_state(bfp);
 
-    txn = probe_txn(bfp);
+    set_txn_mode();
 
-    if (DEBUG_DATABASE(1))
-	fprintf(dbgout, "probing \"%s\" and \"%s\" result %d\n",
-		bfp->dirname, bfp->filename, txn);
-
-    if (txn == P_DISABLE)
-	fTransaction = false;
-    if (txn == P_ENABLE)
-	fTransaction = true;
-
-    /* overrides for test suite etc. */
-    if (getenv("BF_FORCE_NON_TRANSACTIONAL"))
-	fTransaction = false;
-    if (getenv("BF_FORCE_TRANSACTIONAL"))
-	fTransaction = true;
-
-    if (!fTransaction)
+    if (eTransaction == T_DISABLED)
 	dsm = &dsm_traditional;
     else
 	dsm = &dsm_transactional;
 #else
     (void)bfp;
+
+    set_txn_mode();
 #endif
 }
 
@@ -276,36 +331,40 @@ static void handle_free(/*@only@*/ dbh_t *handle)
     return;
 }
 
-static bool checkpath(const char *path, char *norm)
+static bool xstat(const char *path, struct stat *st)
 {
-    if (realpath(path, norm) != NULL)
-	return true;
-    else {
-	print_error(__FILE__, __LINE__,
-		    "error: cannot normalize path \"%s\": %s",
-		    path, strerror(errno));
-	return false;
+    int e = stat(path, st);
+    if (e) {
+	fprintf(stderr, "Cannot stat \"%s\": %s\n",
+		path, strerror(errno));
     }
+    return e;
 }
 
-/** Initialize data base, configure some lock table sizes
- * (which can be overridden in the DB_CONFIG file)
- * and lock the file to tell other parts we're initialized and
- * do not want recovery to stomp over us.
- */
-void *dbe_init(bfpath *bfp)
+/** Ensure that only a single database environment is specified,
+ * \return true for success */
+static bool check_path(bfpath *bfp)
 {
-    char norm_dir[PATH_MAX+1]; /* check normalized directory names */
-    char norm_home[PATH_MAX+1];/* see man realpath(3) for details */
+    struct stat stdn, stbh;
 
-    dbe_t *env;
+    /* Only bogofilter (with transactions) has to worry about multiple environments */
+    if (!fBogofilter || eTransaction == T_DISABLED)
+	return true;
 
-    if ( !checkpath(bfp->dirname, norm_dir) ||
-	 !checkpath(bogohome, norm_home))
-	exit(EX_ERROR);
+    /* If bogohome not yet set, skip check */
+    if (bfp->dirname == NULL || bogohome == NULL)
+	return true;
 
-    if (strcmp(norm_dir, norm_home) != 0)
-    {
+    /* XXX FIXME: If we cannot stat, skip check for now.
+     * We should probably handle this condition but then again when
+     * bootstrapping a setup a missing environment is probably OK */
+    if (xstat(bfp->dirname, &stdn)) return true;
+    if (xstat(bogohome, &stbh)) return true;
+
+    if (stdn.st_dev == stbh.st_dev
+	    && stdn.st_ino == stbh.st_ino)
+	return true;
+    else {
 	fprintf(stderr,
 		"ERROR: only one database _environment_ (directory) can be used at a time.\n"
 		"You CAN use multiple wordlists that are in the same directory.\n\n");
@@ -318,8 +377,24 @@ void *dbe_init(bfpath *bfp)
 		"Please accept our apologies for the inconvenience.\n");
 	fprintf(stderr,
 		"\nAborting program\n");
-	exit(EX_ERROR);
+	return false;
     }
+}
+
+/** Initialize data base, configure some lock table sizes
+ * (which can be overridden in the DB_CONFIG file)
+ * and lock the file to tell other parts we're initialized and
+ * do not want recovery to stomp over us.
+ */
+void *dbe_init(bfpath *bfp)
+{
+    dbe_t *env;
+
+    if (!check_path(bfp))
+	exit(EX_ERROR);
+
+    if (bfp->isdir)
+	return NULL;
 
     dsm_init(bfp);
     
@@ -458,7 +533,8 @@ const char *db_version_str(void)
 #if	!defined(ENABLE_TRANSACTIONS) && !defined(DISABLE_TRANSACTIONS)
     strcat(v, " AUTO-XA");
 #else
-    if (fTransaction)
+    set_txn_mode();
+    if (eTransaction == T_ENABLED)
 	strcat(v, " TRANSACTIONAL");
     else
 	strcat(v, " NON-TRANSACTIONAL");
@@ -529,7 +605,7 @@ void *db_open(void *vhandle,
 	db_file = dsm->dsm_database_name(handle->name);
 
 #ifdef	ENABLE_MEMDEBUG	
-	if (!fTransaction)
+	if (eTransaction == T_DISABLED)
 	    dbp->set_alloc(dbp, md_malloc, md_realloc, md_free);
 	else
 	    dbe->set_alloc(dbe, md_malloc, md_realloc, md_free);
@@ -661,7 +737,7 @@ int db_delete(void *vhandle, const dbv_t *token)
     DBT_init(db_key);
 
     assert(handle->magic == MAGIC_DBH);
-    assert((fTransaction == false) == (handle->txn == NULL));
+    assert((eTransaction == T_DISABLED) == (handle->txn == NULL));
 
     db_key.data = token->data;
     db_key.size = token->leng;
@@ -695,7 +771,7 @@ int db_get_dbvalue(void *vhandle, const dbv_t *token, /*@out@*/ dbv_t *val)
 
     assert(handle);
     assert(handle->magic == MAGIC_DBH);
-    assert((fTransaction == false) == (handle->txn == NULL));
+    assert((eTransaction == T_DISABLED) == (handle->txn == NULL));
 
     DBT_init(db_key);
     DBT_init(db_data);
@@ -751,7 +827,7 @@ int db_set_dbvalue(void *vhandle, const dbv_t *token, const dbv_t *val)
     DB *dbp = handle->dbp;
 
     assert(handle->magic == MAGIC_DBH);
-    assert((fTransaction == false) == (handle->txn == NULL));
+    assert((eTransaction == T_DISABLED) == (handle->txn == NULL));
 
     DBT_init(db_key);
     DBT_init(db_data);
@@ -810,7 +886,7 @@ void db_close(void *vhandle)
     dbh_t *handle = vhandle;
     DB *dbp = handle->dbp;
     /* This is _ONLY_ safe as long as we're logging TXNs */
-    uint32_t flag = fTransaction ? DB_NOSYNC : 0;
+    uint32_t flag = (eTransaction == T_ENABLED) ? DB_NOSYNC : 0;
 
     assert(handle->magic == MAGIC_DBH);
 
@@ -898,7 +974,7 @@ ex_t db_foreach(void *vhandle, db_foreach_t hook, void *userdata)
     dbv_t dbv_key, dbv_data;
 
     assert(handle->magic == MAGIC_DBH);
-    assert((fTransaction == false) == (handle->txn == NULL));
+    assert((eTransaction == T_DISABLED) == (handle->txn == NULL));
 
     memset(&key, 0, sizeof(key));
     memset(&data, 0, sizeof(data));
@@ -979,7 +1055,11 @@ bool is_file_or_missing(const char* path) /*@globals errno,stderr@*/
     return false;
 }
 
-u_int32_t db_pagesize(bfpath *bfp)
+static u_int32_t db_do_getpsize(DB *db) {
+    return get_psize(db, true);
+}
+
+static u_int32_t db_dofile(bfpath *bfp, u_int32_t (*func)(DB *))
 {
     DB_ENV *dbe = NULL;
     DB *db;
@@ -1007,7 +1087,7 @@ u_int32_t db_pagesize(bfpath *bfp)
 		bfp->filename, db_strerror(e));
 	exit(EX_ERROR);
     }
-    s = get_psize(db, true);
+    s = func(db);
 
     e = db->close(db, 0);
     if (e != 0) {
@@ -1027,6 +1107,29 @@ u_int32_t db_pagesize(bfpath *bfp)
     return s;
 }
 
+u_int32_t db_pagesize(bfpath *bfp) {
+    return db_dofile(bfp, db_do_getpsize);
+}
+
+static u_int32_t db_do_leafpages(DB *dbp) {
+    int ret;
+    u_int32_t c;
+    DB_BTREE_STAT *dbstat = NULL;
+
+    ret = BF_DB_STAT(dbp, NULL, &dbstat, 0);
+    if (ret) {
+	print_error(__FILE__, __LINE__, "DB->stat");
+	return 0xffffffff;
+    }
+    c = dbstat->bt_leaf_pg;
+    free(dbstat);
+    return c;
+}
+
+u_int32_t db_leafpages(bfpath *bfp) {
+    return db_dofile(bfp, db_do_leafpages);
+}
+
 ex_t db_verify(bfpath *bfp)
 {
     DB_ENV *dbe = NULL;
@@ -1067,4 +1170,3 @@ ex_t db_verify(bfpath *bfp)
 
     return e;
 }
-
