@@ -1,4 +1,4 @@
-/* $Id: datastore_db_trans.c,v 1.26 2005/04/04 11:16:20 relson Exp $ */
+/* $Id: datastore_db_trans.c,v 1.39 2005/09/04 14:49:43 relson Exp $ */
 
 /*****************************************************************************
 
@@ -14,7 +14,6 @@ David Relson	<relson@osagesoftware.com> 2005
 
 ******************************************************************************/
 
-#define DONT_TYPEDEF_SSIZE_T 1
 #include "common.h"
 
 #include <assert.h>
@@ -40,26 +39,25 @@ David Relson	<relson@osagesoftware.com> 2005
 static int lockfd = -1;	/* fd of lock file to prevent concurrent recovery */
 
 /** Default flags for DB_ENV->open() */
-static const u_int32_t dbenv_defflags = DB_INIT_MPOOL | DB_INIT_LOCK
+static const u_int32_t dbenv_defflags = DB_INIT_MPOOL
 					| DB_INIT_LOG | DB_INIT_TXN;
 
 /* public -- used in datastore.c */
 static int	   dbx_begin		(void *vhandle);
-static int  	   dbx_abort		(void *vhandle);
-static int  	   dbx_commit		(void *vhandle);
+static int	   dbx_abort		(void *vhandle);
+static int	   dbx_commit		(void *vhandle);
 /* private -- used in datastore_db_*.c */
 static DB_ENV	  *dbx_get_env_dbe	(dbe_t *env);
 static const char *dbx_database_name	(const char *db_file);
 static DB_ENV	  *dbx_recover_open	(bfpath *bfp);
 static int	   dbx_auto_commit_flags(void);
 static int	   dbx_get_rmw_flag	(int open_mode);
-static int	   dbx_lock		(void *handle, int open_mode);
 static ex_t	   dbx_common_close	(DB_ENV *dbe, bfpath *bfp);
 static int	   dbx_sync		(DB_ENV *dbe, int ret);
 static void	   dbx_log_flush	(DB_ENV *dbe);
-static dbe_t 	  *dbx_init		(bfpath *bfp);
-static void 	   dbx_cleanup		(dbe_t *env);
-static void 	   dbx_cleanup_lite	(dbe_t *env);
+static dbe_t	  *dbx_init		(bfpath *bfp);
+static void	   dbx_cleanup		(dbe_t *env);
+static void	   dbx_cleanup_lite	(dbe_t *env);
 static ex_t	   dbe_env_purgelogs	(DB_ENV *dbe);
 
 static ex_t	   dbx_checkpoint	(bfpath *bfp);
@@ -67,6 +65,8 @@ static ex_t	   dbx_purgelogs	(bfpath *bfp);
 static ex_t	   dbx_recover		(bfpath *bfp, bool catastrophic, bool force);
 static ex_t	   dbx_remove		(bfpath *bfp);
 
+static ex_t	   dbx_list_logfiles	(bfpath *bfp, int argc, char **argv);
+
 /* OO function lists */
 
 dsm_t dsm_transactional = {
@@ -84,7 +84,7 @@ dsm_t dsm_transactional = {
     &dbx_recover_open,
     &dbx_auto_commit_flags,
     &dbx_get_rmw_flag,
-    &dbx_lock,
+    &db_lock,
     &dbx_common_close,
     &dbx_sync,
     &dbx_log_flush,
@@ -93,7 +93,9 @@ dsm_t dsm_transactional = {
     &dbx_purgelogs,
     &dbx_recover,
     &dbx_remove,
-    &db_verify
+    &db_verify,
+    &dbx_list_logfiles,
+    &db_leafpages
 };
 
 /* non-OO static function prototypes */
@@ -101,9 +103,9 @@ dsm_t dsm_transactional = {
 static int plock(const char *path, short locktype, int mode);
 static int db_try_glock(bfpath *bfp, short locktype, int lockcmd);
 static int bf_dbenv_create(DB_ENV **dbe);
-static void dbe_config(void *vhandle, u_int32_t numlocks, u_int32_t numobjs);
-static dbe_t *dbe_xinit(dbe_t *env, bfpath *bfp, u_int32_t numlocks, u_int32_t numobjs, u_int32_t flags);
-static DB_ENV *dbe_recover_open(bfpath *bfp, uint32_t flags);
+static void dbe_config(void *vhandle);
+static dbe_t *dbe_xinit(dbe_t *env, bfpath *bfp, u_int32_t flags);
+static DB_ENV *dbe_recover_open(bfpath *bfp, u_int32_t flags);
 
 /* support functions */
 
@@ -116,6 +118,16 @@ static bool get_bool(const char *name, const char *arg)
     return b;
 }
 
+static e_txn get_txn(const char *name, const char *arg)
+{
+    e_txn t = get_bool(name, arg) ? T_ENABLED : T_DISABLED;
+    if (DEBUG_CONFIG(2))
+	fprintf(dbgout, "%s -> %s\n", name,
+		t ? "enabled" : "disabled");
+    return t;
+}
+
+
 /* non-OO static functions */
 
 DB_ENV *dbx_get_env_dbe(dbe_t *env)
@@ -143,24 +155,10 @@ int  dbx_auto_commit_flags(void)
 #endif
 }
 
-int dbx_lock(void *vhandle, int open_mode)
-{
-    (void) vhandle;
-    (void) open_mode;
-
-    return 0;
-}
-
 int dbx_get_rmw_flag(int open_mode)
 {
-    int flag;
-
-    if (open_mode == DS_READ)
-	flag = 0;
-    else
-	flag = DB_RMW;
-
-    return flag;
+    (void)open_mode;
+    return 0;
 }
 
 /** print user-readable diagnostics and instructions after DB_ENV->open
@@ -193,9 +191,9 @@ static void diag_dbeopen(
 }
 
 /** run recovery, open environment and keep the exclusive lock */
-static DB_ENV *dbe_recover_open(bfpath *bfp, uint32_t flags)
+static DB_ENV *dbe_recover_open(bfpath *bfp, u_int32_t flags)
 {
-    const uint32_t local_flags = flags | DB_CREATE;
+    const u_int32_t local_flags = flags | DB_CREATE;
     DB_ENV *dbe;
     int e;
 
@@ -370,7 +368,7 @@ static int db_try_glock(bfpath *bfp, short locktype, int lockcmd)
     char *t;
 
     /* lock */
-    ret = mkdir(bfp->dirname, DIR_MODE);
+    ret = bf_mkdir(bfp->dirname, DIR_MODE);
     if (ret && errno != EEXIST) {
 	print_error(__FILE__, __LINE__, "mkdir(%s): %s",
 		bfp->dirname, strerror(errno));
@@ -420,50 +418,12 @@ static int bf_dbenv_create(DB_ENV **env)
     return ret;
 }
 
-static void dbe_config(void *vhandle, u_int32_t numlocks, u_int32_t numobjs)
+static void dbe_config(void *vhandle)
 {
     dbe_t *env = vhandle;
     int ret = 0;
     u_int32_t logsize = 1048576;    /* 1 MByte (default in BDB 10 MByte) */
 
-    /* configure lock system size - locks */
-#if DB_AT_LEAST(3,2)
-    if ((ret = env->dbe->set_lk_max_locks(env->dbe, numlocks)) != 0)
-#else
-    if ((ret = env->dbe->set_lk_max(env->dbe, numlocks)) != 0)
-#endif
-    {
-	print_error(__FILE__, __LINE__, "DB_ENV->set_lk_max_locks(%p, %lu), err: %d, %s", (void *)env->dbe,
-		(unsigned long)numlocks, ret, db_strerror(ret));
-	exit(EX_ERROR);
-    }
-
-    if (DEBUG_DATABASE(1))
-	fprintf(dbgout, "DB_ENV->set_lk_max_locks(%p, %lu)\n", (void *)env->dbe, (unsigned long)numlocks);
-
-#if DB_AT_LEAST(3,2)
-    /* configure lock system size - objects */
-    if ((ret = env->dbe->set_lk_max_objects(env->dbe, numobjs)) != 0) {
-	print_error(__FILE__, __LINE__, "DB_ENV->set_lk_max_objects(%p, %lu), err#%d: %s", (void *)env->dbe,
-		(unsigned long)numobjs, ret, db_strerror(ret));
-	exit(EX_ERROR);
-    }
-    if (DEBUG_DATABASE(1))
-	fprintf(dbgout, "DB_ENV->set_lk_max_objects(%p, %lu)\n", (void *)env->dbe, (unsigned long)numlocks);
-#else
-    /* suppress compiler warning for unused variable */
-    (void)numobjs;
-#endif
-
-    /* configure automatic deadlock detector */
-    if ((ret = env->dbe->set_lk_detect(env->dbe, DB_LOCK_DEFAULT)) != 0) {
-	print_error(__FILE__, __LINE__, "DB_ENV->set_lk_detect(DB_LOCK_DEFAULT), err: %d, %s", ret, db_strerror(ret));
-	exit(EX_ERROR);
-    }
-
-    if (DEBUG_DATABASE(1))
-	fprintf(dbgout, "DB_ENV->set_lk_detect(DB_LOCK_DEFAULT)\n");
-
     /* configure log file size */
     ret = env->dbe->set_lg_max(env->dbe, logsize);
     if (ret) {
@@ -514,15 +474,14 @@ static dbe_t *dbx_init(bfpath *bfp)
 #endif
 #endif
 
-    dbe_xinit(env, bfp, db_max_locks, db_max_objects, flags);
+    dbe_xinit(env, bfp, flags);
 
     return env;
 }
 
 /* dummy infrastructure, to be expanded by environment
  * or transactional initialization/shutdown */
-static dbe_t *dbe_xinit(dbe_t *env, bfpath *bfp,
-	u_int32_t numlocks, u_int32_t numobjs, u_int32_t flags)
+static dbe_t *dbe_xinit(dbe_t *env, bfpath *bfp, u_int32_t flags)
 {
     int ret;
 
@@ -540,7 +499,7 @@ static dbe_t *dbe_xinit(dbe_t *env, bfpath *bfp,
     if (DEBUG_DATABASE(1))
 	fprintf(dbgout, "DB_ENV->set_cachesize(%u)\n", db_cachesize);
 
-    dbe_config(env, numlocks, numobjs);
+    dbe_config(env);
 
     flags |= DB_CREATE | dbenv_defflags;
 
@@ -633,7 +592,7 @@ ex_t dbx_recover(bfpath *bfp, bool catastrophic, bool force)
     dbe_t *env = xcalloc(1, sizeof(dbe_t));
 
     /* set exclusive/write lock for recovery */
-    while((force || needs_recovery())
+    while ((force || needs_recovery())
 	    && (db_try_glock(bfp, F_WRLCK, F_SETLKW) <= 0))
 	rand_sleep(10000,1000000);
 
@@ -645,8 +604,7 @@ ex_t dbx_recover(bfpath *bfp, bool catastrophic, bool force)
     if (DEBUG_DATABASE(0))
         fprintf(dbgout, "running %s data base recovery\n",
 	    catastrophic ? "catastrophic" : "regular");
-    env = dbe_xinit(env, bfp, 
-		    db_max_locks, db_max_objects,
+    env = dbe_xinit(env, bfp,
 		    catastrophic ? DB_RECOVER_FATAL : DB_RECOVER);
     if (env == NULL) {
 	exit(EX_ERROR);
@@ -802,6 +760,8 @@ const char **dsm_help_bogoutil(void)
 	"                                (only effective at creation time)\n",
 	"      --db-verify=file        - verify data file.\n",
 	"      --db-checkpoint=dir     - flush buffer cache and checkpoint database\n",
+	"      --db-list-logfiles=dir [FLAGS]\n"
+	"                              - list logfiles in environment\n",
 	"      --db-prune=dir          - remove inactive log files in dir.\n",
 	"      --db-recover=dir        - run recovery on database in dir.\n",
 	"      --db-recover-harder=dir - run catastrophic recovery on database.\n",
@@ -824,11 +784,10 @@ const char **dsm_help_bogoutil(void)
 bool dsm_options_bogofilter(int option, const char *name, const char *val)
 {
     switch (option) {
-	case O_DB_TRANSACTION:		fTransaction		= get_bool(name, val);			return true;
+	case O_DB_TRANSACTION:		
+	    eTransaction = get_txn(name, val);								return true;
 
 #ifdef	HAVE_DECL_DB_CREATE
-	case O_DB_MAX_OBJECTS:		db_max_objects		= atoi(val);				return true;
-	case O_DB_MAX_LOCKS:		db_max_locks		= atoi(val);				return true;
 	case O_DB_LOG_AUTOREMOVE:	db_log_autoremove	= get_bool(name, val);			return true;
 #ifdef	FUTURE_DB_OPTIONS
 	case O_DB_TXN_DURABLE:		db_txn_durable		= get_bool(name, val);			return true;
@@ -842,7 +801,7 @@ bool dsm_options_bogoutil(int option, cmd_t *flag, int *count, const char **ds_f
 {
     switch (option) {
 	case O_DB_TRANSACTION:
-	    fTransaction = get_bool(name, val);
+	    eTransaction = get_txn(name, val);
 	    return true;
 
 	case O_DB_RECOVER:
@@ -863,6 +822,12 @@ bool dsm_options_bogoutil(int option, cmd_t *flag, int *count, const char **ds_f
 	    *ds_file = val;
 	    return true;
 
+	case O_DB_LIST_LOGFILES:
+	    *flag = M_LIST_LOGFILES;
+	    *count += 1;
+	    *ds_file = val;
+	    return true;
+
 	case O_DB_PRUNE:
 	    *flag = M_PURGELOGS;
 	    *count += 1;
@@ -875,21 +840,10 @@ bool dsm_options_bogoutil(int option, cmd_t *flag, int *count, const char **ds_f
 	    *count += 1;
 	    return true;
 
-#ifdef	HAVE_DECL_DB_CREATE
-	case O_DB_MAX_OBJECTS:	
-	    db_max_objects = atoi(val);
-	    return true;
-	case O_DB_MAX_LOCKS:
-	    db_max_locks   = atoi(val);
-	    return true;
-	case O_DB_LOG_AUTOREMOVE:
-	    db_log_autoremove = get_bool(name, val);
-	    return true;
 #ifdef	FUTURE_DB_OPTIONS
 	case O_DB_TXN_DURABLE:
 	    db_txn_durable    = get_bool(name, val);
 	    return true;
-#endif
 #endif
 
 	default:
@@ -900,7 +854,7 @@ bool dsm_options_bogoutil(int option, cmd_t *flag, int *count, const char **ds_f
 /** probe if the directory contains an environment, and if so,
  * if it has transactions
  */
-probe_txn_t probe_txn(bfpath *bfp)
+e_txn probe_txn(bfpath *bfp)
 {
     DB_ENV *dbe;
     int r;
@@ -912,20 +866,29 @@ probe_txn_t probe_txn(bfpath *bfp)
     if (r) {
 	print_error(__FILE__, __LINE__, "cannot create environment handle: %s",
 		db_strerror(r));
-	return P_ERROR;
+	return T_ERROR;
     }
 
+    /* we might call dbe->set_flags here to set DB_NOPANIC, but this is
+     * only supported from 4.1 onwards and probably not worth the
+     * effort, we'll just check for DB_RUNRECOVERY */
+
 #if DB_AT_LEAST(3,2)
     r = dbe->open(dbe, bfp->dirname, DB_JOINENV, DS_MODE);
 #else
     r = ENOENT;
 #endif
+    if (r == DB_RUNRECOVERY) {
+	dbe->close(dbe, 0);
+	return T_ENABLED;
+    }
+
     if (r == ENOENT) {
 	struct stat st;
 	int w;
 	char *t = bfp->filepath;
 	struct dirent *de;
-	probe_txn_t rc = P_DONT_KNOW;
+	e_txn rc = T_DONT_KNOW;
 	DIR *d;
 
 	/* no environment found by JOINENV, but clean up handle */
@@ -938,27 +901,27 @@ probe_txn_t probe_txn(bfpath *bfp)
 	if (d == NULL) {
 	    print_error(__FILE__, __LINE__, "cannot open directory %s: %s",
 		    t, strerror(r));
-	    rc = P_ERROR;
+	    rc = T_ERROR;
 	} else {
 	    while ((errno = 0, de = readdir(d))) {
 		if (strlen(de->d_name) == 14
 		    && strncmp(de->d_name, "log.", 4) == 0
 		    && strspn(de->d_name + 4, "0123456789") == 10)
 		{
-		    rc = P_ENABLE;
+		    rc = T_ENABLED;
 		    break;
 		}
 	    }
 	    if (errno)
-		rc = P_ERROR;
+		rc = T_ERROR;
 	    closedir(d);
 
-	    if (rc != P_ERROR && rc != P_ENABLE) {
+	    if (rc != T_ERROR && rc != T_ENABLED) {
 		w = stat(t, &st);
 		if (w == 0) {
-		    rc = P_DISABLE;
+		    rc = T_DISABLED;
 		} else if (errno != ENOENT) {
-		    rc = P_ERROR;
+		    rc = T_ERROR;
 		    print_error(__FILE__, __LINE__, "cannot stat %s: %s",
 			    t, db_strerror(r));
 		}
@@ -970,7 +933,7 @@ probe_txn_t probe_txn(bfpath *bfp)
     if (r != 0) {
 	print_error(__FILE__, __LINE__, "cannot join environment: %s",
 		db_strerror(r));
-	return P_ERROR;
+	return T_ERROR;
     }
 
     /* environment found, validate if it has transactions */
@@ -979,17 +942,60 @@ probe_txn_t probe_txn(bfpath *bfp)
     if (r) {
 	print_error(__FILE__, __LINE__, "cannot query flags: %s",
 		db_strerror(r));
-	return P_ERROR;
+	return T_ERROR;
     }
 
     dbe->close(dbe, 0);
     if ((flags & DB_INIT_TXN) == 0) {
 	print_error(__FILE__, __LINE__,
 		"environment found but does not support transactions.");
-	return P_ERROR;
+	return T_ERROR;
     }
 #else
     dbe->close(dbe, 0);
 #endif
-    return P_ENABLE;
+    return T_ENABLED;
+}
+
+static ex_t dbx_list_logfiles(bfpath *bfp, int argc, char **argv)
+{
+    ex_t e;
+    char **list, **i;
+    int j;
+    u_int32_t flags = 0;
+    DB_ENV *dbe = dbe_recover_open(bfp, 0);
+
+    if (dbe == NULL)
+	exit(EX_ERROR);
+
+    for (j = 0; j < argc; j++) {
+	if (strcasecmp(argv[j], "all") == 0)
+	    flags |= DB_ARCH_LOG;
+	if (strcasecmp(argv[j], "absolute") == 0)
+	    flags |= DB_ARCH_ABS;
+    }
+
+    e = BF_LOG_ARCHIVE(dbe, &list, flags);
+    if (e != 0) {
+	print_error(__FILE__, __LINE__,
+		"DB_ENV->log_archive failed: %s",
+		db_strerror(e));
+	exit(EX_ERROR);
+    }
+
+    if (list) {
+	for (i = list; *i; i++) {
+	    if (flags & DB_ARCH_ABS)
+		puts(*i);
+	    else
+		printf("%s%s%s\n", bfp->dirname, DIRSEP_S, *i);
+	}
+    }
+
+    xfree(list);
+    fflush(stdout);
+    e = ferror(stdout) ? EX_ERROR : EX_OK;
+
+    if (dbx_common_close(dbe, bfp)) e = EX_ERROR;
+    return e;
 }
