@@ -1,4 +1,4 @@
-/* $Id: format.c,v 1.42 2004/07/15 00:56:19 m-a Exp $ */
+/* $Id: format.c,v 1.47 2005/09/04 14:49:43 relson Exp $ */
 
 /*****************************************************************************
 
@@ -15,6 +15,8 @@ Most of the ideas in here are stolen from Mutt's snprintf implementation.
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
+#include <pwd.h>
+#include <unistd.h>
 
 #ifdef NEEDTRIO
 #include "trio.h"
@@ -72,6 +74,8 @@ const char *unsure_subject_tag = NULL;			/* used in passthrough mode */
 **	        w - word count
 **	        m - message count
 **
+**	    u - user name, f. i. "login=%u"
+**
 **	    v - version, ex. "version=%v"
 */
 
@@ -81,11 +85,13 @@ const char *log_header_format = "%h: %c, spamicity=%p, version=%v";
 const char *log_update_format = "register-%r, %w words, %m messages";
 
 #define RC_COUNT 3
+#ifdef DEAD_CODE
 static FIELD spamicity_tags_ynu[RC_COUNT] = { "Yes",   "No",    "Unsure" };
+#endif
 static FIELD spamicity_tags_shu[RC_COUNT] = { "Spam",  "Ham",   "Unsure" };
 static FIELD spamicity_format_d[RC_COUNT] = { "%0.6f", "%0.6f", "%0.6f" };
 
-FIELD  *spamicity_tags    = spamicity_tags_ynu;
+FIELD  *spamicity_tags    = spamicity_tags_shu;
 FIELD  *spamicity_formats = spamicity_format_d;
 
 /* Descriptors for config file */
@@ -248,6 +254,7 @@ char *convert_format_to_string(char *buff, size_t size, const char *format)
     char *beg = buff;
     char *end = beg + size;
     char temp[20];
+    char *tptr;
 
     int state = S_DEFAULT;
     int min = 0;
@@ -273,7 +280,7 @@ char *convert_format_to_string(char *buff, size_t size, const char *format)
 	case S_FLAGS:
 	    if ('1' <= *format && *format <= '9')
 		format++;
-	    else if(*format == '-')
+	    else if (*format == '-')
 		format++;
 	    else
 	    switch (*format) {
@@ -319,7 +326,7 @@ char *convert_format_to_string(char *buff, size_t size, const char *format)
 		*buff++ = '%';
 		break;
 	    case 'A':		/* A - Message Address */
-		buff += format_string(buff, msg_addr ? (const char *)msg_addr->text : "UNKNOWN", 0, prec, flags, end);
+		buff += format_string(buff, msg_addr.leng != 0 ? (const char *)msg_addr.text : "UNKNOWN", 0, prec, flags, end);
 		break;
 	    case 'c':		/* c - classification, e.g. Yes/No, Spam/Ham/Unsure, or YN, SHU, +-? */
 	    {
@@ -341,10 +348,10 @@ char *convert_format_to_string(char *buff, size_t size, const char *format)
 		break;
 	    }
 	    case 'I':		/* M - Message ID */
-		buff += format_string(buff, msg_id ? (const char *)msg_id->text : "UNKNOWN", 0, prec, flags, end);
+		buff += format_string(buff, msg_id.leng != 0 ? (const char *)msg_id.text : "UNKNOWN", 0, prec, flags, end);
 		break;
 	    case 'Q':		/* Q - Queue ID */
-		buff += format_string(buff, queue_id ? (const char *)queue_id->text : "UNKNOWN", 0, prec, flags, end);
+		buff += format_string(buff, queue_id.leng != 0 ? (const char *)queue_id.text : "UNKNOWN", 0, prec, flags, end);
 		break;
 	    case 'p':		/* p - spamicity as a probability */
 	    {
@@ -374,6 +381,21 @@ char *convert_format_to_string(char *buff, size_t size, const char *format)
 		snprintf( temp, sizeof(temp), "%s%s", reg, unreg );
 		buff += format_string(buff, temp, 0, 0, 0, end);
 		break;
+	    case 'u':		/* u - user name */
+		{
+		    struct passwd *p;
+		    tptr = getlogin();
+		    if (!tptr) {
+			if ((p = getpwuid(getuid()))) {
+			    tptr = p->pw_name;
+			} else {
+			    snprintf(temp, sizeof(temp), "#%ld", (long)getuid());
+			    tptr = temp;
+			}
+		    }
+		    buff += format_string(buff, tptr, 0, prec, flags, end);
+		}
+		break;
 	    case 'w':		/* w - word count */
 		snprintf( temp, sizeof(temp), "%u", wrdcount );
 		buff += format_string(buff, temp, 0, 0, 0, end);
