@@ -1,4 +1,4 @@
-/* $Id: lexer.h,v 1.43 2004/07/19 13:16:54 relson Exp $ */
+/* $Id: lexer.h,v 1.44 2005/06/25 16:50:38 relson Exp $ */
 
 /*****************************************************************************
 
@@ -70,7 +70,7 @@ extern int	yyinput(byte *buf, size_t used, size_t size);
 
 extern int	buff_fill(buff_t *buff, size_t used, size_t need);
 
-extern size_t	text_decode(word_t *w);
+extern word_t  *text_decode(word_t *w);
 extern size_t	html_decode(word_t *w);
 
 #endif	/* LEXER_H */
