@@ -1,4 +1,4 @@
-/* $Id: longoptions.h,v 1.18 2005/06/18 16:36:09 relson Exp $ */
+/* $Id: longoptions.h,v 1.20 2005/12/27 15:38:12 relson Exp $ */
 
 /*****************************************************************************
 
@@ -28,8 +28,6 @@ typedef enum longopts_e {
     O_BLOCK_ON_SUBNETS = 1000,
     O_CHARSET_DEFAULT,
     O_CONFIG_FILE,
-    O_DB_MAX_OBJECTS,
-    O_DB_MAX_LOCKS,
     O_DB_CHECKPOINT,
     O_DB_LIST_LOGFILES,
     O_DB_PRINT_LEAFPAGE_COUNT,
@@ -81,6 +79,7 @@ typedef enum longopts_e {
     { "no-config-file",			N, 0, 'C' }, \
     { "help",				N, 0, 'h' }, \
     { "input-file",			N, 0, 'I' }, \
+    { "output-file",			N, 0, 'O' }, \
        UNICODE_OPTION \
     { "version",			N, 0, 'V' }, \
     { "verbosity",			N, 0, 'v' },
@@ -89,7 +88,6 @@ typedef enum longopts_e {
 #define LONGOPTIONS_LEX \
     { "block-on-subnets",		R, 0, O_BLOCK_ON_SUBNETS }, \
     { "charset-default",		R, 0, O_CHARSET_DEFAULT }, \
-    { "output-file",			N, 0, 'O' }, \
     { "user-config-file",		R, 0, O_USER_CONFIG_FILE }, \
     { "replace-nonascii-characters",	R, 0, O_REPLACE_NONASCII_CHARACTERS },
 
@@ -102,8 +100,6 @@ typedef enum longopts_e {
 #ifdef	HAVE_DECL_DB_CREATE
  #undef lo1
  #define lo1 \
-    { "db-lk-max-locks",		R, 0, O_DB_MAX_LOCKS }, \
-    { "db-lk-max-objects",		R, 0, O_DB_MAX_OBJECTS }, \
     { "db-log-autoremove",		R, 0, O_DB_LOG_AUTOREMOVE },
  #ifdef	FUTURE_DB_OPTIONS
   #undef lo2
