@@ -1,4 +1,4 @@
-/* $Id: passthrough.c,v 1.33 2004/10/19 01:24:22 m-a Exp $ */
+/* $Id: passthrough.c,v 1.36 2005/12/27 15:38:12 relson Exp $ */
 
 /*****************************************************************************
 
@@ -9,6 +9,7 @@ NAME:
 
 #include "common.h"
 
+#include <assert.h>
 #include <ctype.h>
 #include <errno.h>
 #include <stdlib.h>
@@ -40,6 +41,9 @@ size_t msg_register_size = sizeof(msg_register);
 static void cleanup_exit(ex_t exitcode, int killfiles)
     __attribute__ ((noreturn));
 static void cleanup_exit(ex_t exitcode, int killfiles) {
+    if (fpo) {
+	output_cleanup();
+    }
     if (killfiles && outfname[0] != '\0') unlink(outfname);
     exit(exitcode);
 }
@@ -212,9 +216,8 @@ static void write_body(readfunc_t rf, void *rfarg)
 
     if (!hadlf) (void) fputs(eol, fpo);
 
-    if (fflush(fpo) || ferror(fpo) || (fpo != stdout && fclose(fpo))) {
+    if (fflush(fpo) || ferror(fpo))
 	cleanup_exit(EX_ERROR, 1);
-    }
 }
 
 static void build_spam_header(void)
@@ -307,26 +310,54 @@ void write_log_message(rc_t status)
 #endif
 }
 
+/* we need to take care not to fclose() a shared file descriptor, if,
+ * for instance we've fdopen()ed STDOUT_FILENO, becase we'd prevent
+ * other streams accessing the same file descriptor from flushing their
+ * caches. Prominent symptom: t.bogodir failure.
+ */
+static bool fpo_mayclose;	/* if output_cleanup can close the file */
+
 void output_setup(void)
 {
+    assert(fpo == NULL);
+
     if (*outfname && passthrough) {
-	if ((fpo = fopen(outfname,"wt"))==NULL)
-	{
-	    fprintf(stderr,"Cannot open %s: %s\n",
-		    outfname, strerror(errno));
-	    exit(EX_ERROR);
-	}
+	fpo = fopen(outfname,"wt");
+	fpo_mayclose = true;
     } else {
-	fpo = stdout;
+	fpo = fdopen(STDOUT_FILENO, "wt");
+	fpo_mayclose = false;
     }
 
+    if (fpo == NULL) {
+	if (*outfname)
+	    fprintf(stderr, "Cannot open %s: %s\n",
+		    outfname, strerror(errno));
+	else
+	    fprintf(stderr, "Cannot fdopen STDOUT: %s\n", strerror(errno));
+
+	exit(EX_ERROR);
+    }
+
+    /* if we're not in passthrough mode, set line buffered mode just in
+     * case some program that calls uses waits for our output in -T mode */
     if (!passthrough) {
-	fflush(fpo);
 	mysetvbuf(fpo, NULL, _IOLBF, BUFSIZ);
     }
 }
 
-void passthrough_setup()
+void output_cleanup(void) {
+    int rc;
+
+    rc = fflush(fpo);
+    if (fpo_mayclose)
+	rc |= fclose(fpo);
+    fpo = NULL;
+    if (rc)
+	cleanup_exit(EX_ERROR, 1);
+}
+
+void passthrough_setup(void)
 {
     /* check if the input is seekable, if it is, we don't need to buffer
      * things in memory => configure passmode accordingly
@@ -347,21 +378,21 @@ void passthrough_setup()
 	case PASS_SEEK: m = "rewind and reread file"; break;
 	default:        m = "unknown"; break;
 	}
-	fprintf(fpo, "passthrough mode: %s\n", m);
+	fprintf(dbgout, "passthrough mode: %s\n", m);
     }
 }
 
-int passthrough_keepopen()
+int passthrough_keepopen(void)
 {
     return passthrough && passmode == PASS_SEEK ;
 }
 
-void passthrough_cleanup()
+void passthrough_cleanup(void)
 {
     if (!passthrough)
 	return;
 
-    switch(passmode) {
+    switch (passmode) {
     case PASS_MEM:
 	textblock_free();
 	break;
